/***********************************************************************
      This is an example of a blackbox optimization model which 
      communicates via plain text file input/output
***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
/***********************************************************************/
int main()
{
    /* Problem dimensions
    *********************/
    int o  = 1; /* Number of objectives                          */
    int n  = 4; /* Number of variables (in total)                */
    int m  = 3; /* Number of constraints (in total)              */

    /* Initialize F,G,X arrays for input/ooutput via textfile
    *********************************************************/
    double f[10], g[100], x[1000];

    /*#################################################################*/
    /*###############  READ  X VARIABLES FROM TEXT-FILE  ##############*/
    /*#################################################################*/

    FILE *input_file;
    input_file=fopen("variables.txt", "r");
    if(input_file)
    {
        for(int i=0; i<n; i++)
        { 
            fscanf(input_file, "%lf",&x[i]);
        }
        fclose(input_file);
    }else
    { 
        printf("\n ERROR IN BLACKBOX: NO  variables.txt  FILE FOUND");
        printf("\n [Pause]"); getchar();
        return 0;
    }

    /*#################################################################*/
    /*#########  CALCULATE MODEL OBJECTIVE AND CONSTRAINTS  ###########*/
    /*#################################################################*/

    /* Objective functions F(X) */
    f[0] = (x[0]-1.0)*(x[0]-1.0)
         + (x[1]-2.0)*(x[1]-2.0)
         + (x[2]-3.0)*(x[2]-3.0)
         + (x[3]-4.0)*(x[3]-4.0)
         + 1.23456789;

    /* Equality constraints G(X) = 0 MUST COME FIRST in g[0:me-1] */
    g[0] = x[0] - 1.0; 
    /* Inequality constraints G(X) >= 0 MUST COME SECOND in g[me:m-1] */
    g[1] = x[1] - 1.333333333;
    g[2] = x[2] - 2.666666666;

    /*
       Multi-Objective Functions F(X)
    */
    /* f[0] = (x[0]-0.0)*(x[0]-0.0) + (x[1]-0.0)*(x[1]-0.0); */
    /* f[1] = (x[0]-1.0)*(x[0]-1.0) + (x[1]-1.0)*(x[1]-1.0); */

    /*#################################################################*/
    /*########  WRITE OBJECTIVE AND CONSTRAINTS TO TEXT-FILE  #########*/
    /*#################################################################*/

    FILE *output_file;
    output_file=fopen("objectives.txt", "w+");    
    for(int i=0; i<o; i++){ fprintf(output_file, " %32.16e \n",f[i]); } /* OBJECTIVES F */
    for(int i=0; i<m; i++){ fprintf(output_file, " %32.16e \n",g[i]); } /* CONSTRAINTS G */
    fclose(output_file);

    return 0;
}